@@ -343,8 +343,8 @@ assets.fetchFilterList = async function(mainlistURL) {
 
 **/
 
-let assetSourceRegistryPromise,
-    assetSourceRegistry = Object.create(null);
+let assetSourceRegistryPromise;
+let assetSourceRegistry = Object.create(null);
 
 const getAssetSourceRegistry = function() {
     if ( assetSourceRegistryPromise === undefined ) {
@@ -434,7 +434,7 @@ const saveAssetSourceRegistry = (( ) => {
     };
 })();
 
-const updateAssetSourceRegistry = function(json, silent) {
+const updateAssetSourceRegistry = function(json, silent = false) {
     let newDict;
     try {
         newDict = JSON.parse(json);
@@ -538,6 +538,7 @@ const saveAssetCacheRegistry = (( ) => {
 })();
 
 const assetCacheRead = async function(assetKey, updateReadTime = false) {
+    const t0 = Date.now();
     const internalKey = `cache/${assetKey}`;
 
     const reportBack = function(content) {
@@ -551,6 +552,12 @@ const assetCacheRead = async function(assetKey, updateReadTime = false) {
         getAssetCacheRegistry(),
         cacheStorage.get(internalKey),
     ]);
+
+    µb.supportStats.maxAssetCacheWait = Math.max(
+        Date.now() - t0,
+        parseInt(µb.supportStats.maxAssetCacheWait, 10)
+    ) + ' ms';
+
     if (
         bin instanceof Object === false ||
         bin.hasOwnProperty(internalKey) === false
