@@ -344,7 +344,7 @@ const processLoggerEntries = function(response) {
 /******************************************************************************/
 
 const parseLogEntry = function(details) {
-    // Patch realm until changed all over codebase to make this unecessary
+    // Patch realm until changed all over codebase to make this unnecessary
     if ( details.realm === 'cosmetic' ) {
         details.realm = 'extended';
     }
@@ -490,7 +490,7 @@ const viewPort = (( ) => {
         vwContent.style.top = `${lastTopPix}px`;
     };
 
-    // Coallesce scroll events
+    // Coalesce scroll events
     const onScroll = function() {
         if ( scrollTimer !== undefined ) { return; }
         scrollTimer = setTimeout(
@@ -1005,7 +1005,7 @@ const onLogBufferRead = function(response) {
     processLoggerEntries(response);
 
     // Synchronize DOM with sent logger data
-    document.body.classList.toggle(
+    document.documentElement.classList.toggle(
         'colorBlind',
         response.colorBlind === true
     );
@@ -1384,7 +1384,7 @@ const reloadTab = function(ev) {
             return;
         }
 
-        // Hightlight corresponding element in target web page
+        // Highlight corresponding element in target web page
         if ( tcl.contains('picker') ) {
             ev.stopPropagation();
             messaging.send('loggerUI', {
