@@ -19,10 +19,49 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/******************************************************************************/
+'use strict';
+
 /******************************************************************************/
 
-'use strict';
+import publicSuffixList from '../lib/publicsuffixlist/publicsuffixlist.js';
+import punycode from '../lib/punycode.js';
+
+import cacheStorage from './cachestorage.js';
+import cosmeticFilteringEngine from './cosmetic-filtering.js';
+import htmlFilteringEngine from './html-filtering.js';
+import logger from './logger.js';
+import lz4Codec from './lz4.js';
+import io from './assets.js';
+import scriptletFilteringEngine from './scriptlet-filtering.js';
+import staticExtFilteringEngine from './static-ext-filtering.js';
+import staticFilteringReverseLookup from './reverselookup.js';
+import staticNetFilteringEngine from './static-net-filtering.js';
+import µb from './background.js';
+import webRequest from './traffic.js';
+import { denseBase64 } from './base64-custom.js';
+import { redirectEngine } from './redirect-engine.js';
+import { StaticFilteringParser } from './static-filtering-parser.js';
+
+import {
+    permanentFirewall,
+    sessionFirewall,
+    permanentSwitches,
+    sessionSwitches,
+    permanentURLFiltering,
+    sessionURLFiltering,
+} from './filtering-engines.js';
+
+import {
+    domainFromHostname,
+    domainFromURI,
+    entityFromDomain,
+    hostnameFromURI,
+    isNetworkURI,
+} from './uri-utils.js';
+
+import './benchmarks.js';
+
+/******************************************************************************/
 
 // https://github.com/uBlockOrigin/uBlock-issues/issues/710
 //   Listeners have a name and a "privileged" status.
@@ -34,13 +73,11 @@
 /******************************************************************************/
 
 // Default handler
-//      priviledged
+//      privileged
 
 {
 // >>>>> start of local scope
 
-const µb = µBlock;
-
 const clickToLoad = function(request, sender) {
     const { tabId, frameId } = sender;
     if ( tabId === undefined || frameId === undefined ) { return false; }
@@ -51,12 +88,11 @@ const clickToLoad = function(request, sender) {
 };
 
 const getDomainNames = function(targets) {
-    const µburi = µb.URI;
     return targets.map(target => {
         if ( typeof target !== 'string' ) { return ''; }
         return target.indexOf('/') !== -1
-            ? µburi.domainFromURI(target) || ''
-            : µburi.domainFromHostname(target) || target;
+            ? domainFromURI(target) || ''
+            : domainFromHostname(target) || target;
     });
 };
 
@@ -65,7 +101,7 @@ const onMessage = function(request, sender, callback) {
     switch ( request.what ) {
     case 'getAssetContent':
         // https://github.com/chrisaljoudi/uBlock/issues/417
-        µb.assets.get(request.url, {
+        io.get(request.url, {
             dontCache: true,
             needSourceURL: true,
         }).then(result => {
@@ -74,7 +110,7 @@ const onMessage = function(request, sender, callback) {
         return;
 
     case 'listsFromNetFilter':
-        µb.staticFilteringReverseLookup.fromNetFilter(
+        staticFilteringReverseLookup.fromNetFilter(
             request.rawFilter
         ).then(response => {
             callback(response);
@@ -82,7 +118,7 @@ const onMessage = function(request, sender, callback) {
         return;
 
     case 'listsFromCosmeticFilter':
-        µb.staticFilteringReverseLookup.fromCosmeticFilter(
+        staticFilteringReverseLookup.fromCosmeticFilter(
             request
         ).then(response => {
             callback(response);
@@ -94,11 +130,15 @@ const onMessage = function(request, sender, callback) {
         return;
 
     case 'scriptlet':
-        µb.scriptlets.inject(request.tabId, request.scriptlet, callback);
+        vAPI.tabs.executeScript(request.tabId, {
+            file: `/js/scriptlets/${request.scriptlet}.js`
+        }).then(result => {
+            callback(result);
+        });
         return;
 
-    case 'sfneBenchmark':
-        µb.staticNetFilteringEngine.benchmark().then(result => {
+    case 'snfeBenchmark':
+        µb.benchmarkStaticNetFiltering({ redirectEngine }).then(result => {
             callback(result);
         });
         return;
@@ -125,7 +165,7 @@ const onMessage = function(request, sender, callback) {
 
     case 'forceUpdateAssets':
         µb.scheduleAssetUpdater(0);
-        µb.assets.updateStart({
+        io.updateStart({
             delay: µb.hiddenSettings.manualUpdateAssetFetchPeriod
         });
         break;
@@ -179,25 +219,39 @@ const onMessage = function(request, sender, callback) {
         µb.toggleHostnameSwitch(request);
         break;
 
+    case 'uiAccentStylesheet':
+        µb.uiAccentStylesheet = request.stylesheet;
+        break;
+
     case 'uiStyles':
         response = {
+            uiAccentCustom: µb.userSettings.uiAccentCustom,
+            uiAccentCustom0: µb.userSettings.uiAccentCustom0,
+            uiAccentStylesheet: µb.uiAccentStylesheet,
             uiStyles: µb.hiddenSettings.uiStyles,
-            uiTheme: vAPI.webextFlavor.soup.has('devbuild')
-                ? µb.hiddenSettings.uiTheme
-                : 'unset',
+            uiTheme: µb.userSettings.uiTheme,
         };
         break;
 
     case 'userSettings':
         response = µb.changeUserSettings(request.name, request.value);
-        if (
-            vAPI.net.canUncloakCnames !== true &&
-            response instanceof Object
-        ) {
-            response.cnameUncloakEnabled = undefined;
+        if ( response instanceof Object ) {
+            if ( vAPI.net.canUncloakCnames !== true ) {
+                response.cnameUncloakEnabled = undefined;
+            }
+            response.canLeakLocalIPAddresses =
+                vAPI.browserSettings.canLeakLocalIPAddresses === true;
         }
         break;
 
+    case 'snfeDump':
+        response = staticNetFilteringEngine.dump();
+        break;
+
+    case 'cfeDump':
+        response = cosmeticFilteringEngine.dump();
+        break;
+
     default:
         return vAPI.messaging.UNHANDLED;
     }
@@ -220,8 +274,6 @@ vAPI.messaging.setup(onMessage);
 {
 // >>>>> start of local scope
 
-const µb = µBlock;
-
 const createCounts = ( ) => {
     return {
         blocked: { any: 0, frame: 0, script: 0 },
@@ -244,7 +296,7 @@ const getHostnameDict = function(hostnameDetailsMap, out) {
     for ( const hnDetails of hostnameDetailsMap.values() ) {
         const hostname = hnDetails.hostname;
         if ( hnDict[hostname] !== undefined ) { continue; }
-        const domain = vAPI.domainFromHostname(hostname) || hostname;
+        const domain = domainFromHostname(hostname) || hostname;
         const dnDetails =
             hostnameDetailsMap.get(domain) || { counts: createCounts() };
         if ( hnDict[domain] === undefined ) {
@@ -270,7 +322,7 @@ const firewallRuleTypes = [
 
 const getFirewallRules = function(src, out) {
     const ruleset = out.firewallRules = {};
-    const df = µb.sessionFirewall;
+    const df = sessionFirewall;
 
     for ( const type of firewallRuleTypes ) {
         const r = df.lookupRuleData('*', '*', type);
@@ -325,7 +377,7 @@ const popupDataFromTabId = function(tabId, tabTitle) {
         tooltipsDisabled: µbus.tooltipsDisabled
     };
 
-    if ( µbhs.uiPopupConfig !== 'undocumented' ) {
+    if ( µbhs.uiPopupConfig !== 'unset' ) {
         r.uiPopupConfig = µbhs.uiPopupConfig;
     }
 
@@ -336,27 +388,27 @@ const popupDataFromTabId = function(tabId, tabTitle) {
         getHostnameDict(pageStore.getAllHostnameDetails(), r);
         r.contentLastModified = pageStore.contentLastModified;
         getFirewallRules(rootHostname, r);
-        r.canElementPicker = µb.URI.isNetworkURI(r.rawURL);
-        r.noPopups = µb.sessionSwitches.evaluateZ(
+        r.canElementPicker = isNetworkURI(r.rawURL);
+        r.noPopups = sessionSwitches.evaluateZ(
             'no-popups',
             rootHostname
         );
         r.popupBlockedCount = pageStore.popupBlockedCount;
-        r.noCosmeticFiltering = µb.sessionSwitches.evaluateZ(
+        r.noCosmeticFiltering = sessionSwitches.evaluateZ(
             'no-cosmetic-filtering',
             rootHostname
         );
-        r.noLargeMedia = µb.sessionSwitches.evaluateZ(
+        r.noLargeMedia = sessionSwitches.evaluateZ(
             'no-large-media',
             rootHostname
         );
         r.largeMediaCount = pageStore.largeMediaCount;
-        r.noRemoteFonts = µb.sessionSwitches.evaluateZ(
+        r.noRemoteFonts = sessionSwitches.evaluateZ(
             'no-remote-fonts',
             rootHostname
         );
         r.remoteFontCount = pageStore.remoteFontCount;
-        r.noScripting = µb.sessionSwitches.evaluateZ(
+        r.noScripting = sessionSwitches.evaluateZ(
             'no-scripting',
             rootHostname
         );
@@ -365,14 +417,14 @@ const popupDataFromTabId = function(tabId, tabTitle) {
         getFirewallRules(undefined, r);
     }
 
-    r.matrixIsDirty = µb.sessionFirewall.hasSameRules(
-        µb.permanentFirewall,
+    r.matrixIsDirty = sessionFirewall.hasSameRules(
+        permanentFirewall,
         rootHostname,
         r.hostnameDict
     ) === false;
     if ( r.matrixIsDirty === false ) {
-        r.matrixIsDirty = µb.sessionSwitches.hasSameRules(
-            µb.permanentSwitches,
+        r.matrixIsDirty = sessionSwitches.hasSameRules(
+            permanentSwitches,
             rootHostname
         ) === false;
     }
@@ -439,27 +491,36 @@ const onMessage = function(request, sender, callback) {
 
     // Sync
     let response;
-    let pageStore;
 
     switch ( request.what ) {
-    case 'hasPopupContentChanged':
-        pageStore = µb.pageStoreFromTabId(request.tabId);
-        var lastModified = pageStore ? pageStore.contentLastModified : 0;
+    case 'hasPopupContentChanged': {
+        const pageStore = µb.pageStoreFromTabId(request.tabId);
+        const lastModified = pageStore ? pageStore.contentLastModified : 0;
         response = lastModified !== request.contentLastModified;
         break;
-
+    }
+    case 'launchReporter': {
+        const pageStore = µb.pageStoreFromTabId(request.tabId);
+        if ( pageStore === null ) { break; }
+        const supportURL = new URL(vAPI.getURL('support.html'));
+        supportURL.searchParams.set('pageURL', request.pageURL);
+        supportURL.searchParams.set('popupPanel', request.popupPanel);
+        µb.openNewTab({ url: supportURL.href, select: true, index: -1 });
+        break;
+    }
     case 'revertFirewallRules':
-        µb.sessionFirewall.copyRules(
-            µb.permanentFirewall,
+        // TODO: use Set() to message around sets of hostnames
+        sessionFirewall.copyRules(
+            permanentFirewall,
             request.srcHostname,
-            request.desHostnames
+            Object.assign(Object.create(null), request.desHostnames)
         );
-        µb.sessionSwitches.copyRules(
-            µb.permanentSwitches,
+        sessionSwitches.copyRules(
+            permanentSwitches,
             request.srcHostname
         );
         // https://github.com/gorhill/uBlock/issues/188
-        µb.cosmeticFilteringEngine.removeFromSelectorCache(
+        cosmeticFilteringEngine.removeFromSelectorCache(
             request.srcHostname,
             'net'
         );
@@ -468,18 +529,19 @@ const onMessage = function(request, sender, callback) {
         break;
 
     case 'saveFirewallRules':
+        // TODO: use Set() to message around sets of hostnames
         if (
-            µb.permanentFirewall.copyRules(
-                µb.sessionFirewall,
+            permanentFirewall.copyRules(
+                sessionFirewall,
                 request.srcHostname,
-                request.desHostnames
+                Object.assign(Object.create(null), request.desHostnames)
             )
         ) {
             µb.savePermanentFirewallRules();
         }
         if (
-            µb.permanentSwitches.copyRules(
-                µb.sessionSwitches,
+            permanentSwitches.copyRules(
+                sessionSwitches,
                 request.srcHostname
             )
         ) {
@@ -497,8 +559,8 @@ const onMessage = function(request, sender, callback) {
         response = popupDataFromTabId(request.tabId);
         break;
 
-    case 'toggleNetFiltering':
-        pageStore = µb.pageStoreFromTabId(request.tabId);
+    case 'toggleNetFiltering': {
+        const pageStore = µb.pageStoreFromTabId(request.tabId);
         if ( pageStore ) {
             pageStore.toggleNetFilteringSwitch(
                 request.url,
@@ -508,7 +570,7 @@ const onMessage = function(request, sender, callback) {
             µb.updateToolbarIcon(request.tabId, 0b111);
         }
         break;
-
+    }
     default:
         return vAPI.messaging.UNHANDLED;
     }
@@ -535,8 +597,6 @@ vAPI.messaging.listen({
 {
 // >>>>> start of local scope
 
-const µb = µBlock;
-
 const retrieveContentScriptParameters = async function(sender, request) {
     if ( µb.readyToFilter !== true ) { return; }
     const { tabId, frameId } = sender;
@@ -554,7 +614,6 @@ const retrieveContentScriptParameters = async function(sender, request) {
         request.url = pageStore.getEffectiveFrameURL(sender);
     }
 
-    const loggerEnabled = µb.logger.enabled;
     const noSpecificCosmeticFiltering =
         pageStore.shouldApplySpecificCosmeticFilters(frameId) === false;
     const noGenericCosmeticFiltering =
@@ -568,18 +627,18 @@ const retrieveContentScriptParameters = async function(sender, request) {
 
     request.tabId = tabId;
     request.frameId = frameId;
-    request.hostname = µb.URI.hostnameFromURI(request.url);
-    request.domain = µb.URI.domainFromHostname(request.hostname);
-    request.entity = µb.URI.entityFromDomain(request.domain);
+    request.hostname = hostnameFromURI(request.url);
+    request.domain = domainFromHostname(request.hostname);
+    request.entity = entityFromDomain(request.domain);
 
     response.specificCosmeticFilters =
-        µb.cosmeticFilteringEngine.retrieveSpecificSelectors(request, response);
+        cosmeticFilteringEngine.retrieveSpecificSelectors(request, response);
 
     // The procedural filterer's code is loaded only when needed and must be
     // present before returning response to caller.
     if (
         Array.isArray(response.specificCosmeticFilters.proceduralFilters) || (
-            loggerEnabled &&
+            logger.enabled &&
             response.specificCosmeticFilters.exceptedFilters.length !== 0
         )
     ) {
@@ -597,16 +656,16 @@ const retrieveContentScriptParameters = async function(sender, request) {
     //   effective URL is available here in `request.url`.
     if (
         µb.canInjectScriptletsNow === false ||
-        µb.URI.isNetworkURI(sender.frameURL) === false
+        isNetworkURI(sender.frameURL) === false
     ) {
-        response.scriptlets = µb.scriptletFilteringEngine.retrieve(request);
+        response.scriptlets = scriptletFilteringEngine.retrieve(request);
     }
 
     // https://github.com/NanoMeow/QuickReports/issues/6#issuecomment-414516623
     //   Inject as early as possible to make the cosmetic logger code less
     //   sensitive to the removal of DOM nodes which may match injected
     //   cosmetic filters.
-    if ( loggerEnabled ) {
+    if ( logger.enabled ) {
         if (
             noSpecificCosmeticFiltering === false ||
             noGenericCosmeticFiltering === false
@@ -645,7 +704,7 @@ const onMessage = function(request, sender, callback) {
 
     switch ( request.what ) {
     case 'cosmeticFiltersInjected':
-        µb.cosmeticFilteringEngine.addToSelectorCache(request);
+        cosmeticFilteringEngine.addToSelectorCache(request);
         break;
 
     case 'getCollapsibleBlockedRequests':
@@ -653,7 +712,7 @@ const onMessage = function(request, sender, callback) {
             id: request.id,
             hash: request.hash,
             netSelectorCacheCountMax:
-                µb.cosmeticFilteringEngine.netSelectorCacheCountMax,
+                cosmeticFilteringEngine.netSelectorCacheCountMax,
         };
         if (
             µb.userSettings.collapseBlocked &&
@@ -684,7 +743,7 @@ const onMessage = function(request, sender, callback) {
         request.tabId = sender.tabId;
         request.frameId = sender.frameId;
         response = {
-            result: µb.cosmeticFilteringEngine.retrieveGenericSelectors(request),
+            result: cosmeticFilteringEngine.retrieveGenericSelectors(request),
         };
         break;
 
@@ -714,8 +773,6 @@ vAPI.messaging.listen({
 // >>>>> start of local scope
 
 const onMessage = function(request, sender, callback) {
-    const µb = µBlock;
-
     // Async
     switch ( request.what ) {
     // The procedural filterer must be present in case the user wants to
@@ -780,7 +837,7 @@ const fromBase64 = function(encoded) {
     }
     let u8array;
     try {
-        u8array = µBlock.denseBase64.decode(encoded);
+        u8array = denseBase64.decode(encoded);
     } catch(ex) {
     }
     return Promise.resolve(u8array !== undefined ? u8array : encoded);
@@ -788,22 +845,22 @@ const fromBase64 = function(encoded) {
 
 const toBase64 = function(data) {
     const value = data instanceof Uint8Array
-        ? µBlock.denseBase64.encode(data)
+        ? denseBase64.encode(data)
         : data;
     return Promise.resolve(value);
 };
 
 const compress = function(json) {
-    return µBlock.lz4Codec.encode(json, toBase64);
+    return lz4Codec.encode(json, toBase64);
 };
 
 const decompress = function(encoded) {
-    return µBlock.lz4Codec.decode(encoded, fromBase64);
+    return lz4Codec.decode(encoded, fromBase64);
 };
 
 const onMessage = function(request, sender, callback) {
     // Cloud storage support is optional.
-    if ( µBlock.cloudStorageSupported !== true ) {
+    if ( µb.cloudStorageSupported !== true ) {
         callback();
         return;
     }
@@ -812,7 +869,7 @@ const onMessage = function(request, sender, callback) {
     switch ( request.what ) {
     case 'cloudGetOptions':
         vAPI.cloud.getOptions(function(options) {
-            options.enabled = µBlock.userSettings.cloudStorageEnabled === true;
+            options.enabled = µb.userSettings.cloudStorageEnabled === true;
             callback(options);
         });
         return;
@@ -828,7 +885,7 @@ const onMessage = function(request, sender, callback) {
         });
 
     case 'cloudPush':
-        if ( µBlock.hiddenSettings.cloudStorageCompression ) {
+        if ( µb.hiddenSettings.cloudStorageCompression ) {
             request.encode = compress;
         }
         return vAPI.cloud.push(request).then(result => {
@@ -880,8 +937,6 @@ vAPI.messaging.listen({
 {
 // >>>>> start of local scope
 
-const µb = µBlock;
-
 // Settings
 const getLocalData = async function() {
     const data = Object.assign({}, µb.restoreBackupSettings);
@@ -903,9 +958,9 @@ const backupUserData = async function() {
         hiddenSettings:
             µb.getModifiedSettings(µb.hiddenSettings, µb.hiddenSettingsDefault),
         whitelist: µb.arrayFromWhitelist(µb.netWhitelist),
-        dynamicFilteringString: µb.permanentFirewall.toString(),
-        urlFilteringString: µb.permanentURLFiltering.toString(),
-        hostnameSwitchesString: µb.permanentSwitches.toString(),
+        dynamicFilteringString: permanentFirewall.toString(),
+        urlFilteringString: permanentURLFiltering.toString(),
+        hostnameSwitchesString: permanentSwitches.toString(),
         userFilters: userFilters.content,
     };
 
@@ -941,17 +996,17 @@ const restoreUserData = async function(request) {
 
     // https://github.com/chrisaljoudi/uBlock/issues/1102
     //   Ensure all currently cached assets are flushed from storage AND memory.
-    µb.assets.rmrf();
+    io.rmrf();
 
     // If we are going to restore all, might as well wipe out clean local
     // storages
     await Promise.all([
-        µb.cacheStorage.clear(),
+        cacheStorage.clear(),
         vAPI.storage.clear(),
     ]);
 
     // Restore block stats
-    µBlock.saveLocalSettings();
+    µb.saveLocalSettings();
 
     // Restore user data
     vAPI.storage.set(userData.userSettings);
@@ -959,7 +1014,7 @@ const restoreUserData = async function(request) {
     // Restore advanced settings.
     let hiddenSettings = userData.hiddenSettings;
     if ( hiddenSettings instanceof Object === false ) {
-        hiddenSettings = µBlock.hiddenSettingsFromString(
+        hiddenSettings = µb.hiddenSettingsFromString(
             userData.hiddenSettingsString || ''
         );
     }
@@ -1006,7 +1061,7 @@ const restoreUserData = async function(request) {
 // quite attached to numbers
 const resetUserData = async function() {
     await Promise.all([
-        µb.cacheStorage.clear(),
+        cacheStorage.clear(),
         vAPI.storage.clear(),
     ]);
 
@@ -1017,16 +1072,15 @@ const resetUserData = async function() {
 
 // Filter lists
 const prepListEntries = function(entries) {
-    const µburi = µb.URI;
     for ( const k in entries ) {
         if ( entries.hasOwnProperty(k) === false ) { continue; }
         const entry = entries[k];
         if ( typeof entry.supportURL === 'string' && entry.supportURL !== '' ) {
-            entry.supportName = µburi.hostnameFromURI(entry.supportURL);
+            entry.supportName = hostnameFromURI(entry.supportURL);
         } else if ( typeof entry.homeURL === 'string' && entry.homeURL !== '' ) {
-            const hn = µburi.hostnameFromURI(entry.homeURL);
+            const hn = hostnameFromURI(entry.homeURL);
             entry.supportURL = `http://${hn}/`;
-            entry.supportName = µburi.domainFromHostname(hn);
+            entry.supportName = domainFromHostname(hn);
         }
     }
 };
@@ -1036,17 +1090,18 @@ const getLists = async function(callback) {
         autoUpdate: µb.userSettings.autoUpdate,
         available: null,
         cache: null,
-        cosmeticFilterCount: µb.cosmeticFilteringEngine.getFilterCount(),
+        cosmeticFilterCount: cosmeticFilteringEngine.getFilterCount(),
         current: µb.availableFilterLists,
         ignoreGenericCosmeticFilters: µb.userSettings.ignoreGenericCosmeticFilters,
-        isUpdating: µb.assets.isUpdating(),
-        netFilterCount: µb.staticNetFilteringEngine.getFilterCount(),
+        isUpdating: io.isUpdating(),
+        netFilterCount: staticNetFilteringEngine.getFilterCount(),
         parseCosmeticFilters: µb.userSettings.parseAllABPHideFilters,
+        suspendUntilListsAreLoaded: µb.userSettings.suspendUntilListsAreLoaded,
         userFiltersPath: µb.userFiltersPath
     };
     const [ lists, metadata ] = await Promise.all([
         µb.getAvailableLists(),
-        µb.assets.metadata(),
+        io.metadata(),
     ]);
     r.available = lists;
     prepListEntries(r.available);
@@ -1059,7 +1114,6 @@ const getLists = async function(callback) {
 
 // TODO: also return origin of embedded frames?
 const getOriginHints = function() {
-    const punycode = self.punycode;
     const out = new Set();
     for ( const tabId of µb.pageStores.keys() ) {
         if ( tabId === -1 ) { continue; }
@@ -1079,29 +1133,29 @@ const getOriginHints = function() {
 const getRules = function() {
     return {
         permanentRules:
-            µb.permanentFirewall.toArray().concat(
-                µb.permanentSwitches.toArray(),
-                µb.permanentURLFiltering.toArray()
+            permanentFirewall.toArray().concat(
+                permanentSwitches.toArray(),
+                permanentURLFiltering.toArray()
             ),
         sessionRules:
-            µb.sessionFirewall.toArray().concat(
-                µb.sessionSwitches.toArray(),
-                µb.sessionURLFiltering.toArray()
+            sessionFirewall.toArray().concat(
+                sessionSwitches.toArray(),
+                sessionURLFiltering.toArray()
             ),
-        pslSelfie: self.publicSuffixList.toSelfie(),
+        pslSelfie: publicSuffixList.toSelfie(),
     };
 };
 
 const modifyRuleset = function(details) {
     let swRuleset, hnRuleset, urlRuleset;
     if ( details.permanent ) {
-        swRuleset = µb.permanentSwitches;
-        hnRuleset = µb.permanentFirewall;
-        urlRuleset = µb.permanentURLFiltering;
+        swRuleset = permanentSwitches;
+        hnRuleset = permanentFirewall;
+        urlRuleset = permanentURLFiltering;
     } else {
-        swRuleset = µb.sessionSwitches;
-        hnRuleset = µb.sessionFirewall;
-        urlRuleset = µb.sessionURLFiltering;
+        swRuleset = sessionSwitches;
+        hnRuleset = sessionFirewall;
+        urlRuleset = sessionURLFiltering;
     }
     let toRemove = new Set(details.toRemove.trim().split(/\s*[\n\r]+\s*/));
     for ( let rule of toRemove ) {
@@ -1139,7 +1193,7 @@ const modifyRuleset = function(details) {
     }
 };
 
-// Shortcuts pane
+// Shortcuts
 const getShortcuts = function(callback) {
     if ( µb.canUseShortcuts === false ) {
         return callback([]);
@@ -1173,6 +1227,151 @@ const setShortcut = function(details) {
     vAPI.storage.set({ commandShortcuts: Array.from(µb.commandShortcuts) });
 };
 
+// Support
+const getSupportData = async function() {
+    const diffArrays = function(modified, original) {
+        const modifiedSet = new Set(modified);
+        const originalSet = new Set(original);
+        let added = [];
+        let removed = [];
+        for ( const item of modifiedSet ) {
+            if ( originalSet.has(item) ) { continue; }
+            added.push(item);
+        }
+        for ( const item of originalSet ) {
+            if ( modifiedSet.has(item) ) { continue; }
+            removed.push(item);
+        }
+        if ( added.length === 0 ) {
+            added = undefined;
+        }
+        if ( removed.length === 0 ) {
+            removed = undefined;
+        }
+        if ( added !== undefined || removed !== undefined ) {
+            return { added, removed };
+        }
+    };
+
+    const modifiedUserSettings = µb.getModifiedSettings(
+        µb.userSettings,
+        µb.userSettingsDefault
+    );
+
+    const modifiedHiddenSettings = µb.getModifiedSettings(
+        µb.hiddenSettings,
+        µb.hiddenSettingsDefault
+    );
+
+    let filterset = [];
+    const userFilters = await µb.loadUserFilters();
+    for ( const line of userFilters.content.split(/\s*\n+\s*/) ) {
+        if ( /^($|![^#])/.test(line) ) { continue; }
+        filterset.push(line);
+    }
+
+    const lists = µb.availableFilterLists;
+    let defaultListset = {};
+    let addedListset = {};
+    let removedListset = {};
+    for ( const listKey in lists ) {
+        if ( lists.hasOwnProperty(listKey) === false ) { continue; }
+        const list = lists[listKey];
+        if ( list.content !== 'filters' ) { continue; }
+        const used = µb.selectedFilterLists.includes(listKey);
+        const listDetails = [];
+        if ( used ) {
+            if ( typeof list.entryCount === 'number' ) {
+                listDetails.push(`${list.entryCount}-${list.entryCount-list.entryUsedCount}`);
+            }
+            if ( typeof list.writeTime !== 'number' || list.writeTime === 0 ) {
+                listDetails.push('never');
+            } else {
+                const delta = (Date.now() - list.writeTime) / 1000 | 0;
+                const days = (delta / 86400) | 0;
+                const hours = (delta % 86400) / 3600 | 0;
+                const minutes = (delta % 3600) / 60 | 0;
+                const parts = [];
+                if ( days > 0 ) { parts.push(`${days}d`); }
+                if ( hours > 0 ) { parts.push(`${hours}h`); }
+                if ( minutes > 0 ) { parts.push(`${minutes}m`); }
+                if ( parts.length === 0 ) { parts.push('now'); }
+                listDetails.push(parts.join('.'));
+            }
+        }
+        if ( list.isDefault || listKey === µb.userFiltersPath ) {
+            if ( used ) {
+                defaultListset[listKey] = listDetails.join(', ');
+            } else {
+                removedListset[listKey] = null;
+            }
+        } else if ( used ) {
+            addedListset[listKey] = listDetails.join(', ');
+        }
+    }
+    if ( Object.keys(defaultListset).length === 0 ) {
+        defaultListset = undefined;
+    }
+    if ( Object.keys(addedListset).length === 0 ) {
+        addedListset = undefined;
+    } else if ( Object.keys(addedListset).length > 20 ) {
+        const added = Object.keys(addedListset);
+        const truncated = added.slice(20);
+        for ( const key of truncated ) {
+            delete addedListset[key];
+        }
+        addedListset[`[${truncated.length} lists not shown]`] = '[too many]';
+    }
+    if ( Object.keys(removedListset).length === 0 ) {
+        removedListset = undefined;
+    }
+
+    let browserFamily = (( ) => {
+        if ( vAPI.webextFlavor.soup.has('firefox') ) { return 'Firefox'; }
+        if ( vAPI.webextFlavor.soup.has('chromium') ) { return 'Chromium'; }
+        return 'Unknown';
+    })();
+    if ( vAPI.webextFlavor.soup.has('mobile') ) {
+        browserFamily += ' Mobile';
+    }
+
+    return {
+        [`${vAPI.app.name}`]: `${vAPI.app.version}`,
+        [`${browserFamily}`]: `${vAPI.webextFlavor.major}`,
+        'filterset (summary)': {
+            network: staticNetFilteringEngine.getFilterCount(),
+            cosmetic: cosmeticFilteringEngine.getFilterCount(),
+            scriptlet: scriptletFilteringEngine.getFilterCount(),
+            html: htmlFilteringEngine.getFilterCount(),
+        },
+        'listset (total-discarded, last updated)': {
+            removed: removedListset,
+            added: addedListset,
+            default: defaultListset,
+        },
+        'filterset (user)': filterset,
+        trustedset: diffArrays(
+            µb.arrayFromWhitelist(µb.netWhitelist),
+            µb.netWhitelistDefault
+        ),
+        switchRuleset: diffArrays(
+            sessionSwitches.toArray(),
+            µb.hostnameSwitchesDefault
+        ),
+        hostRuleset: diffArrays(
+            sessionFirewall.toArray(),
+            µb.dynamicFilteringDefault
+        ),
+        urlRuleset: diffArrays(
+            sessionURLFiltering.toArray(),
+            []
+        ),
+        modifiedUserSettings,
+        modifiedHiddenSettings,
+        supportStats: µb.supportStats,
+    };
+};
+
 const onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
@@ -1192,6 +1391,13 @@ const onMessage = function(request, sender, callback) {
     case 'getShortcuts':
         return getShortcuts(callback);
 
+    case 'getSupportData': {
+        getSupportData().then(response => {
+            callback(response);
+        });
+        return;
+    }
+
     case 'readUserFilters':
         return µb.loadUserFilters().then(result => {
             callback(result);
@@ -1220,7 +1426,7 @@ const onMessage = function(request, sender, callback) {
     case 'getAutoCompleteDetails':
         response = {};
         if ( (request.hintUpdateToken || 0) === 0 ) {
-            response.redirectResources = µb.redirectEngine.getResourceDetails();
+            response.redirectResources = redirectEngine.getResourceDetails();
             response.preparseDirectiveTokens = µb.preparseDirectives.getTokens();
             response.preparseDirectiveHints = µb.preparseDirectives.getHints();
             response.expertMode = µb.hiddenSettings.filterAuthorMode;
@@ -1237,22 +1443,22 @@ const onMessage = function(request, sender, callback) {
 
     case 'modifyRuleset':
         // https://github.com/chrisaljoudi/uBlock/issues/772
-        µb.cosmeticFilteringEngine.removeFromSelectorCache('*');
+        cosmeticFilteringEngine.removeFromSelectorCache('*');
         modifyRuleset(request);
         response = getRules();
         break;
 
     case 'purgeAllCaches':
         if ( request.hard ) {
-            µb.assets.remove(/./);
+            io.remove(/./);
         } else {
-            µb.assets.purge(/./, 'public_suffix_list.dat');
+            io.purge(/./, 'public_suffix_list.dat');
         }
         break;
 
     case 'purgeCache':
-        µb.assets.purge(request.assetKey);
-        µb.assets.remove('compiled/' + request.assetKey);
+        io.purge(request.assetKey);
+        io.remove('compiled/' + request.assetKey);
         break;
 
     case 'readHiddenSettings':
@@ -1305,7 +1511,6 @@ vAPI.messaging.listen({
 {
 // >>>>> start of local scope
 
-const µb = µBlock;
 const extensionOriginURL = vAPI.getURL('');
 const documentBlockedURL = vAPI.getURL('document-blocked.html');
 
@@ -1313,7 +1518,7 @@ const getLoggerData = async function(details, activeTabId, callback) {
     const response = {
         activeTabId,
         colorBlind: µb.userSettings.colorBlindFriendly,
-        entries: µb.logger.readAll(details.ownerId),
+        entries: logger.readAll(details.ownerId),
         filterAuthorMode: µb.hiddenSettings.filterAuthorMode,
         tabIdsToken: µb.pageStoresToken,
         tooltips: µb.userSettings.tooltipsDisabled === false
@@ -1366,8 +1571,8 @@ const getURLFilteringData = function(details) {
         dirty: false,
         colors: colors
     };
-    const suf = µb.sessionURLFiltering;
-    const puf = µb.permanentURLFiltering;
+    const suf = sessionURLFiltering;
+    const puf = permanentURLFiltering;
     const urls = details.urls;
     const context = details.context;
     const type = details.type;
@@ -1393,10 +1598,10 @@ const getURLFilteringData = function(details) {
 };
 
 const compileTemporaryException = function(filter) {
-    const parser = new vAPI.StaticFilteringParser();
+    const parser = new StaticFilteringParser();
     parser.analyze(filter);
     if ( parser.shouldDiscard() ) { return; }
-    return µb.staticExtFilteringEngine.compileTemporary(parser);
+    return staticExtFilteringEngine.compileTemporary(parser);
 };
 
 const toggleTemporaryException = function(details) {
@@ -1423,8 +1628,8 @@ const onMessage = function(request, sender, callback) {
     switch ( request.what ) {
     case 'readAll':
         if (
-            µb.logger.ownerId !== undefined &&
-            µb.logger.ownerId !== request.ownerId
+            logger.ownerId !== undefined &&
+            logger.ownerId !== request.ownerId
         ) {
             return callback({ unavailable: true });
         }
@@ -1446,14 +1651,14 @@ const onMessage = function(request, sender, callback) {
         break;
 
     case 'releaseView':
-        if ( request.ownerId === µb.logger.ownerId ) {
-            µb.logger.ownerId = undefined;
+        if ( request.ownerId === logger.ownerId ) {
+            logger.ownerId = undefined;
         }
         break;
 
     case 'saveURLFilteringRules':
-        response = µb.permanentURLFiltering.copyRules(
-            µb.sessionURLFiltering,
+        response = permanentURLFiltering.copyRules(
+            sessionURLFiltering,
             request.context,
             request.urls,
             request.type
@@ -1519,7 +1724,7 @@ const onMessage = function(request, sender, callback) {
         break;
 
     case 'temporarilyWhitelistDocument':
-        µBlock.webRequest.strictBlockBypass(request.hostname);
+        webRequest.strictBlockBypass(request.hostname);
         break;
 
     default:
@@ -1548,10 +1753,8 @@ vAPI.messaging.listen({
 {
 // >>>>> start of local scope
 
-const µb = µBlock;
-
 const logCosmeticFilters = function(tabId, details) {
-    if ( µb.logger.enabled === false ) { return; }
+    if ( logger.enabled === false ) { return; }
 
     const filter = { source: 'cosmetic', raw: '' };
     const fctxt = µb.filteringContext.duplicate();
@@ -1568,7 +1771,7 @@ const logCosmeticFilters = function(tabId, details) {
 };
 
 const logCSPViolations = function(pageStore, request) {
-    if ( µb.logger.enabled === false || pageStore === null ) {
+    if ( logger.enabled === false || pageStore === null ) {
         return false;
     }
     if ( request.violations.length === 0 ) {
@@ -1586,7 +1789,7 @@ const logCSPViolations = function(pageStore, request) {
         cspData = new Map();
 
         const staticDirectives =
-            µb.staticNetFilteringEngine.matchAndFetchModifiers(fctxt, 'csp');
+            staticNetFilteringEngine.matchAndFetchModifiers(fctxt, 'csp');
         if ( staticDirectives !== undefined ) {
             for ( const directive of staticDirectives ) {
                 if ( directive.result !== 1 ) { continue; }
@@ -1671,7 +1874,7 @@ const onMessage = function(request, sender, callback) {
 
     switch ( request.what ) {
     case 'inlinescriptFound':
-        if ( µb.logger.enabled && pageStore !== null ) {
+        if ( logger.enabled && pageStore !== null ) {
             const fctxt = µb.filteringContext.duplicate();
             fctxt.fromTabId(tabId)
                 .setType('inline-script')
@@ -1698,6 +1901,8 @@ const onMessage = function(request, sender, callback) {
         break;
 
     case 'subscribeTo':
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/1797
+        if ( /^(file|https?):\/\//.test(request.location) === false ) { break; }
         const url = encodeURIComponent(request.location);
         const title = encodeURIComponent(request.title);
         const hash = µb.selectedFilterLists.indexOf(request.location) !== -1
