@@ -1136,7 +1136,9 @@ const webRequest = {
 
     start: (( ) => {
         vAPI.net = new vAPI.Net();
-        vAPI.net.suspend();
+        if ( vAPI.Net.canSuspend() ) {
+            vAPI.net.suspend();
+        }
 
         return async ( ) => {
             vAPI.net.setSuspendableListener(onBeforeRequest);
@@ -1146,22 +1148,7 @@ const webRequest = {
                 { urls: [ 'http://*/*', 'https://*/*' ] },
                 [ 'blocking', 'responseHeaders' ]
             );
-            vAPI.net.unsuspend(true);
-            // Mitigation: force-reload active tabs for environments not
-            // supporting suspended network request listeners.
-            if (
-                vAPI.net.canSuspend() !== true ||
-                µb.hiddenSettings.suspendTabsUntilReady === 'no'
-            ) {
-                const tabs = await vAPI.tabs.query({
-                    active: true,
-                    url: [ 'https://*/*', 'http://*/*' ],
-                    windowType: 'normal',
-                });
-                for ( const tab of tabs ) {
-                    vAPI.tabs.reload(tab.id);
-                }
-            }
+            vAPI.net.unsuspend({ all: true });
         };
     })(),
 
