@@ -36,26 +36,19 @@ const uDom = (( ) => {
 
 /******************************************************************************/
 
-const DOMList = function() {
-    this.nodes = [];
+const DOMList = class {
+    constructor() {
+        this.nodes = [];
+    }
+    get length() {
+        return this.nodes.length;
+    }
 };
 
 /******************************************************************************/
 
-Object.defineProperty(
-    DOMList.prototype,
-    'length',
-    {
-        get: function() {
-            return this.nodes.length;
-        }
-    }
-);
-
-/******************************************************************************/
-
 const DOMListFactory = function(selector, context) {
-    var r = new DOMList();
+    const r = new DOMList();
     if ( typeof selector === 'string' ) {
         selector = selector.trim();
         if ( selector !== '' ) {
@@ -74,6 +67,127 @@ const DOMListFactory = function(selector, context) {
     return r;
 };
 
+DOMListFactory.root = document.querySelector(':root');
+
+/******************************************************************************/
+
+DOMListFactory.setTheme = function(theme, propagate = false) {
+    if ( theme === 'auto' ) {
+        if ( typeof self.matchMedia === 'function' ) {
+            const mql = self.matchMedia('(prefers-color-scheme: dark)');
+            theme = mql instanceof Object && mql.matches === true
+                ? 'dark'
+                : 'light';
+        } else {
+            theme = 'light';
+        }
+    }
+    let w = self;
+    for (;;) {
+        const rootcl = w.document.documentElement.classList;
+        if ( theme === 'dark' ) {
+            rootcl.add('dark');
+            rootcl.remove('light');
+        } else /* if ( theme === 'light' ) */ {
+            rootcl.add('light');
+            rootcl.remove('dark');
+        }
+        if ( propagate === false ) { break; }
+        if ( w === w.parent ) { break; }
+        w = w.parent;
+        try { void w.document; } catch(ex) { return; }
+    }
+};
+
+DOMListFactory.setAccentColor = function(
+    accentEnabled,
+    accentColor,
+    propagate,
+    stylesheet = ''
+) {
+    if ( accentEnabled && stylesheet === '' && self.hsluv !== undefined ) {
+        const toRGB = hsl => self.hsluv.hsluvToRgb(hsl).map(a => Math.round(a * 255)).join(' ');
+        // Normalize first
+        const hsl = self.hsluv.hexToHsluv(accentColor);
+        hsl[0] = Math.round(hsl[0] * 10) / 10;
+        hsl[1] = Math.round(Math.min(100, Math.max(0, hsl[1])));
+        // Use normalized result to derive all shades
+        const shades = [ 5, 10, 20, 30, 40, 50, 60, 70, 80, 90, 95 ];
+        const text = [];
+        text.push(':root.accented {');
+        for ( const shade of shades ) {
+            hsl[2] = shade;
+            text.push(`   --primary-${shade}: ${toRGB(hsl)};`);
+        }
+        text.push('}');
+        hsl[1] = Math.min(25, hsl[1]);
+        hsl[2] = 80;
+        text.push(
+            ':root.light.accented {',
+            `    --button-surface-rgb: ${toRGB(hsl)};`,
+            '}',
+        );
+        hsl[2] = 30;
+        text.push(
+            ':root.dark.accented {',
+            `    --button-surface-rgb: ${toRGB(hsl)};`,
+            '}',
+        );
+        text.push('');
+        stylesheet = text.join('\n');
+        vAPI.messaging.send('uDom', { what: 'uiAccentStylesheet', stylesheet });
+    }
+    let w = self;
+    for (;;) {
+        const wdoc = w.document;
+        let style = wdoc.querySelector('style#accentColors');
+        if ( style !== null ) { style.remove(); }
+        if ( accentEnabled ) {
+            style = wdoc.createElement('style');
+            style.id = 'accentColors';
+            style.textContent = stylesheet;
+            wdoc.head.append(style);
+            wdoc.documentElement.classList.add('accented');
+        } else {
+            wdoc.documentElement.classList.remove('accented');
+        }
+        if ( propagate === false ) { break; }
+        if ( w === w.parent ) { break; }
+        w = w.parent;
+        try { void w.document; } catch(ex) { break; }
+    }
+};
+
+{
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1044
+    //   Offer the possibility to bypass uBO's default styling
+    vAPI.messaging.send('uDom', { what: 'uiStyles' }).then(response => {
+        if ( typeof response !== 'object' || response === null ) { return; }
+        uDom.setTheme(response.uiTheme);
+        if ( response.uiAccentCustom ) {
+            uDom.setAccentColor(
+                true,
+                response.uiAccentCustom0,
+                false,
+                response.uiAccentStylesheet
+            );
+        }
+        if ( response.uiStyles !== 'unset' ) {
+            document.body.style.cssText = response.uiStyles;
+        }
+    });
+
+    const rootcl = DOMListFactory.root.classList;
+    if ( vAPI.webextFlavor.soup.has('mobile') ) {
+        rootcl.add('mobile');
+    } else {
+        rootcl.add('desktop');
+    }
+    if ( window.matchMedia('(min-resolution: 150dpi)').matches ) {
+        rootcl.add('hidpi');
+    }
+}
+
 /******************************************************************************/
 
 DOMListFactory.onLoad = function(callback) {
@@ -92,40 +206,6 @@ DOMListFactory.nodeFromSelector = function(selector) {
 
 /******************************************************************************/
 
-{
-    // https://github.com/uBlockOrigin/uBlock-issues/issues/1044
-    //   Offer the possibility to bypass uBO's default styling
-    vAPI.messaging.send('uDom', { what: 'uiStyles' }).then(response => {
-        if ( typeof response !== 'object' || response === null ) { return; }
-        if ( response.uiTheme !== 'unset' ) {
-            if ( response.uiTheme === 'light' ) {
-                root.classList.remove('dark');
-            } else if ( response.uiTheme === 'dark' ) {
-                root.classList.add('dark');
-            }
-        }
-        if ( response.uiStyles !== 'unset' ) {
-            document.body.style.cssText = response.uiStyles;
-        }
-    });
-
-    const root = DOMListFactory.root = document.querySelector(':root');
-    if ( vAPI.webextFlavor.soup.has('mobile') ) {
-        root.classList.add('mobile');
-    } else {
-        root.classList.add('desktop');
-    }
-    if ( window.matchMedia('(min-resolution: 150dpi)').matches ) {
-        root.classList.add('hidpi');
-    }
-    // TODO: re-enable once there is a fully functional dark theme 
-    //if ( window.matchMedia('(prefers-color-scheme: dark)').matches ) {
-    //    root.classList.add('dark');
-    //}
-}
-
-/******************************************************************************/
-
 const addNodeToList = function(list, node) {
     if ( node ) {
         list.nodes.push(node);
@@ -552,18 +632,11 @@ DOMList.prototype.text = function(text) {
 /******************************************************************************/
 
 const toggleClass = function(node, className, targetState) {
-    var tokenList = node.classList;
-    if ( tokenList instanceof DOMTokenList === false ) {
-        return;
-    }
-    var currentState = tokenList.contains(className);
-    var newState = targetState;
-    if ( newState === undefined ) {
-        newState = !currentState;
-    }
-    if ( newState === currentState ) {
-        return;
-    }
+    const tokenList = node.classList;
+    if ( tokenList instanceof DOMTokenList === false ) { return; }
+    const currentState = tokenList.contains(className);
+    const newState = targetState !== undefined ? targetState : !currentState;
+    if ( newState === currentState ) { return; }
     tokenList.toggle(className, newState);
 };
 
@@ -573,7 +646,7 @@ DOMList.prototype.hasClass = function(className) {
     if ( !this.nodes.length ) {
         return false;
     }
-    var tokenList = this.nodes[0].classList;
+    const tokenList = this.nodes[0].classList;
     return tokenList instanceof DOMTokenList &&
            tokenList.contains(className);
 };
@@ -587,9 +660,8 @@ DOMList.prototype.removeClass = function(className) {
     if ( className !== undefined ) {
         return this.toggleClass(className, false);
     }
-    var i = this.nodes.length;
-    while ( i-- ) {
-        this.nodes[i].className = '';
+    for ( const node of this.nodes ) {
+        node.className = '';
     }
     return this;
 };
@@ -600,9 +672,8 @@ DOMList.prototype.toggleClass = function(className, targetState) {
     if ( className.indexOf(' ') !== -1 ) {
         return this.toggleClasses(className, targetState);
     }
-    var i = this.nodes.length;
-    while ( i-- ) {
-        toggleClass(this.nodes[i], className, targetState);
+    for ( const node of this.nodes ) {
+        toggleClass(node, className, targetState);
     }
     return this;
 };
@@ -610,14 +681,10 @@ DOMList.prototype.toggleClass = function(className, targetState) {
 /******************************************************************************/
 
 DOMList.prototype.toggleClasses = function(classNames, targetState) {
-    var tokens = classNames.split(/\s+/);
-    var i = this.nodes.length;
-    var node, j;
-    while ( i-- ) {
-        node = this.nodes[i];
-        j = tokens.length;
-        while ( j-- ) {
-            toggleClass(node, tokens[j], targetState);
+    const tokens = classNames.split(/\s+/);
+    for ( const node of this.nodes ) {
+        for ( const token of tokens ) {
+            toggleClass(node, token, targetState);
         }
     }
     return this;
