@@ -2,6 +2,8 @@
 #
 # This script assumes a linux environment
 
+set -e
+
 echo "*** uBlock0.opera: Creating web store package"
 
 DES=dist/build/uBlock0.opera
@@ -13,12 +15,12 @@ bash ./tools/copy-common-files.sh  $DES
 
 # Chromium-specific
 echo "*** uBlock0.opera: Copying chromium-specific files"
-cp platform/chromium/*.js          $DES/js/
-cp platform/chromium/*.html        $DES/
+cp platform/chromium/*.js   $DES/js/
+cp platform/chromium/*.html $DES/
 
 # Opera-specific
 echo "*** uBlock0.opera: Copying opera-specific files"
-cp platform/opera/manifest.json    $DES/
+cp platform/opera/manifest.json $DES/
 
 rm -r $DES/_locales/az
 rm -r $DES/_locales/cv
@@ -27,6 +29,7 @@ rm -r $DES/_locales/hy
 rm -r $DES/_locales/ka
 rm -r $DES/_locales/kk
 rm -r $DES/_locales/mr
+rm -r $DES/_locales/so
 rm -r $DES/_locales/th
 
 # Removing WASM modules until I receive an answer from Opera people: Opera's
