@@ -2,6 +2,8 @@
 #
 # This script assumes a linux environment
 
+set -e
+
 echo "*** uBlock0.thunderbird: Creating web store package"
 
 BLDIR=dist/build
@@ -12,11 +14,14 @@ mkdir -p $DES
 echo "*** uBlock0.thunderbird: copying common files"
 bash ./tools/copy-common-files.sh  $DES
 
-cp -R $DES/_locales/nb                 $DES/_locales/no
+echo "*** uBlock0.firefox: Copying firefox-specific files"
+cp platform/firefox/*.js $DES/js/
 
-cp platform/thunderbird/manifest.json  $DES/
-cp platform/firefox/webext.js          $DES/js/
-cp platform/firefox/vapi-webrequest.js $DES/js/
+echo "*** uBlock0.firefox: Copying thunderbird-specific files"
+cp platform/thunderbird/manifest.json $DES/
+
+# Firefox store-specific
+cp -R $DES/_locales/nb             $DES/_locales/no
 
 # Firefox/webext-specific
 rm $DES/img/icon_128.png
